@@ -3,8 +3,11 @@ from typing import Optional, Sequence
 import numpy
 from arbol.arbol import aprint, asection
 from joblib import Parallel, delayed
+from toolz import curry
 
 from dexp.datasets import BaseDataset
+from dexp.datasets.stack_iterator import StackIterator
+from dexp.datasets.zarr_dataset import ZDataset
 from dexp.processing.registration.model.model_io import from_json
 from dexp.processing.registration.model.sequence_registration_model import (
     SequenceRegistrationModel,
@@ -18,7 +21,6 @@ from dexp.utils.misc import compute_num_workers
 def _compute_model(
     input_dataset: BaseDataset,
     channel: str,
-    slicing,
     max_range: int,
     min_confidence: float,
     enable_com: bool,
@@ -35,15 +37,15 @@ def _compute_model(
     maxproj: bool = True,
     device: int = 0,
     workers: int = 1,
-    debug_output=None,
 ) -> SequenceRegistrationModel:
 
     # get channel array:
     array = input_dataset.get_array(channel, per_z_slice=False, wrap_with_dask=True)
 
     # Perform slicing:
-    if slicing is not None:
-        array = array[slicing]
+    if input_dataset.slicing is not None:
+        # dexp slicing API doesn't work with dask
+        array = array[input_dataset.slicing]
 
     # Shape and chunks for array:
     ndim = array.ndim
@@ -71,7 +73,6 @@ def _compute_model(
                 edge_filter=edge_filter,
                 internal_dtype=numpy.float16,
                 workers=workers,
-                debug_output=debug_output,
             )
             model.to_numpy()
 
@@ -79,8 +80,8 @@ def _compute_model(
         projections = []
         for axis in range(array.ndim - 1):
             projection = input_dataset.get_projection_array(channel=channel, axis=axis, wrap_with_dask=False)
-            if slicing is not None:
-                projection = projection[slicing]
+            if input_dataset.slicing is not None:
+                projection = projection[input_dataset.slicing]
 
             proj_axis = list(1 + a for a in range(array.ndim - 1) if a != axis)
             projections.append((*proj_axis, projection))
@@ -103,7 +104,6 @@ def _compute_model(
                 edge_filter=edge_filter,
                 ndim=ndim - 1,
                 internal_dtype=numpy.float16,
-                debug_output=debug_output,
                 detrend=detrend,
             )
             model.to_numpy()
@@ -111,18 +111,39 @@ def _compute_model(
     return model
 
 
+# definition of function that processes each time point:
+@curry
+def _process(
+    i: int,
+    array: StackIterator,
+    output_dataset: ZDataset,
+    channel: str,
+    model: SequenceRegistrationModel,
+    pad: bool,
+    integral: bool,
+) -> None:
+    with asection(f"Processing time point: {i}/{len(array)}"):
+        with NumpyBackend() as bkd:
+            with asection("Loading stack"):
+                tp_array = bkd.to_backend(array[i])
+
+                with asection("Applying model..."):
+                    tp_array = model.apply(tp_array, index=i, pad=pad, integral=integral)
+
+            with asection(
+                f"Saving stabilized stack for time point {i}/{len(array)}, shape:{tp_array.shape}, "
+                + "dtype:{array.dtype}"
+            ):
+                output_dataset.write_stack(channel=channel, time_point=i, stack_array=tp_array)
+
+
 def dataset_stabilize(
-    dataset: BaseDataset,
-    output_path: str,
+    input_dataset: BaseDataset,
+    output_dataset: ZDataset,
     channels: Sequence[str],
     model_output_path: str,
     model_input_path: Optional[str] = None,
     reference_channel: Optional[str] = None,
-    slicing=None,
-    zarr_store: str = "dir",
-    compression_codec: str = "zstd",
-    compression_level: int = 3,
-    overwrite: bool = False,
     max_range: int = 7,
     min_confidence: float = 0.5,
     enable_com: bool = False,
@@ -142,9 +163,6 @@ def dataset_stabilize(
     workers: int = -1,
     workers_backend: str = "threading",
     device: int = 0,
-    check: bool = True,
-    stop_at_exception: bool = True,
-    debug_output=None,
 ):
     """
     Takes an input dataset and performs image stabilisation and outputs a stabilised dataset
@@ -152,19 +170,14 @@ def dataset_stabilize(
 
     Parameters
     ----------
-    dataset: Input dataset
-    output_path: Output path for Zarr storage
+    input_dataset: Input dataset
+    output_dataset: Output dataset
     channels: selected channels
     model_output_path: str
         Path to store computed stabilization model for reproducibility.
     model_input_path: Optional[str] = None,
         Path of previously computed stabilization model.
     reference_channel: use this channel to compute the stabilization model and apply to every channel.
-    slicing: selected array slicing
-    zarr_store: type of store, can be 'dir', 'ndir', or 'zip'
-    compression_codec: compression codec to be used ('zstd', 'blosclz', 'lz4', 'lz4hc', 'zlib' or 'snappy').
-    compression_level: An integer between 0 and 9 specifying the compression level.
-    overwrite: overwrite output dataset if already exists
     max_range: maximal distance, in time points, between pairs of images to registrate.
     min_confidence: minimal confidence below which pairwise registrations are rejected for the stabilisation.
     enable_com: enable center of mass fallback when standard registration fails.
@@ -186,30 +199,22 @@ def dataset_stabilize(
     workers: number of workers, if -1 then the number of workers == number of cores or devices
     workers_backend: What backend to spawn workers with, can be ‘loky’ (multi-process) or ‘threading’ (multi-thread)
     device: Sets the CUDA devices id, e.g. 0,1,2
-    check: Checking integrity of written file.
-    stop_at_exception: True to stop as soon as there is an exception during processing.
     """
 
     if model_input_path is not None and reference_channel is not None:
         raise ValueError("`model_input_path` and `reference channel` cannot be supplied at the same time.")
 
-    from dexp.datasets import ZDataset
-
-    mode = "w" + ("" if overwrite else "-")
-    dest_dataset = ZDataset(output_path, mode, zarr_store, parent=dataset)
-
     model = None
     if model_input_path is not None:
         with open(model_input_path) as f:
             model = from_json(f.read())
 
     elif reference_channel is not None:
-        if reference_channel not in dataset.channels():
+        if reference_channel not in input_dataset.channels():
             raise ValueError(f"Reference channel {reference_channel} not found.")
         model = _compute_model(
-            input_dataset=dataset,
+            input_dataset=input_dataset,
             channel=reference_channel,
-            slicing=slicing,
             max_range=max_range,
             min_confidence=min_confidence,
             enable_com=enable_com,
@@ -226,17 +231,15 @@ def dataset_stabilize(
             maxproj=maxproj,
             device=device,
             workers=workers,
-            debug_output=debug_output,
         )
         with open(model_output_path, mode="w") as f:
             f.write(model.to_json())
 
-    for channel in dataset._selected_channels(channels):
+    for channel in input_dataset._selected_channels(channels):
         if model is None:
             channel_model = _compute_model(
-                input_dataset=dataset,
+                input_dataset=input_dataset,
                 channel=channel,
-                slicing=slicing,
                 max_range=max_range,
                 min_confidence=min_confidence,
                 enable_com=enable_com,
@@ -253,7 +256,6 @@ def dataset_stabilize(
                 maxproj=maxproj,
                 device=device,
                 workers=workers,
-                debug_output=debug_output,
             )
             formatted_path = f"{model_output_path.split('.')[0]}_{channel}.json"
             with open(formatted_path, mode="w") as f:
@@ -262,7 +264,7 @@ def dataset_stabilize(
         else:
             channel_model = model
 
-        metadata = dataset.get_metadata()
+        metadata = input_dataset.get_metadata()
         if "dt" in metadata.get(channel, {}):
             prev_displacement = channel_model.total_displacement()
             channel_model = channel_model.reduce(int(round(metadata[channel]["dt"])))
@@ -276,12 +278,9 @@ def dataset_stabilize(
                 # minimum displacement different results in translations shift from reference channel
                 metadata[channel][name] = current[0] - prev[0]
                 aprint(f"{name.upper()}: {current[0] - prev[0]}")
-            dest_dataset.append_metadata(metadata)
-
-        array = dataset.get_array(channel, per_z_slice=False, wrap_with_dask=True)
-        if slicing is not None:
-            array = array[slicing]
+            output_dataset.append_metadata(metadata)
 
+        array = input_dataset[channel]
         shape = array.shape
         dtype = array.dtype
         nb_timepoints = shape[0]
@@ -297,37 +296,17 @@ def dataset_stabilize(
 
         # Shape of the resulting array:
         padded_shape = (nb_timepoints,) + channel_model.padded_shape(shape[1:])
+        output_dataset.add_channel(name=channel, shape=padded_shape, dtype=dtype)
 
-        dest_dataset.add_channel(
-            name=channel, shape=padded_shape, dtype=dtype, codec=compression_codec, clevel=compression_level
+        process = _process(
+            array=array,
+            output_dataset=output_dataset,
+            channel=channel,
+            model=channel_model,
+            pad=pad,
+            integral=integral,
         )
 
-        # definition of function that processes each time point:
-        def process(tp):
-            try:
-                with asection(f"Processing time point: {tp}/{nb_timepoints} ."):
-                    with asection("Loading stack"):
-                        tp_array = array[tp].compute()
-
-                    with NumpyBackend():
-                        with asection("Applying model..."):
-                            tp_array = channel_model.apply(tp_array, index=tp, pad=pad, integral=integral)
-
-                    with asection(
-                        f"Saving stabilized stack for time point {tp}/{nb_timepoints}, shape:{tp_array.shape}, "
-                        + "dtype:{array.dtype}"
-                    ):
-                        dest_dataset.write_stack(channel=channel, time_point=tp, stack_array=tp_array)
-
-            except Exception as error:
-                aprint(error)
-                aprint(f"Error occurred while processing time point {tp}/{nb_timepoints} !")
-                import traceback
-
-                traceback.print_exc()
-                if stop_at_exception:
-                    raise error
-
         # start jobs:
         if workers == 1:
             for tp in range(0, nb_timepoints):
@@ -336,17 +315,8 @@ def dataset_stabilize(
             n_jobs = compute_num_workers(workers, nb_timepoints)
             Parallel(n_jobs=n_jobs, backend=workers_backend)(delayed(process)(tp) for tp in range(0, nb_timepoints))
 
-    # printout output dataset info:
-    aprint(dest_dataset.info())
-    if check:
-        dest_dataset.check_integrity()
-
     # Dataset info:
-    aprint(dest_dataset.info())
+    aprint(output_dataset.info())
 
     # Check dataset integrity:
-    if check:
-        dest_dataset.check_integrity()
-
-    # close destination dataset:
-    dest_dataset.close()
+    output_dataset.check_integrity()
