@@ -2,10 +2,8 @@ import numpy
 from skimage.data import camera
 from skimage.util import random_noise
 
-from dexp.processing.backends.backend import Backend
-from dexp.processing.backends.cupy_backend import CupyBackend
-from dexp.processing.backends.numpy_backend import NumpyBackend
 from dexp.processing.filters.butterworth_filter import butterworth_filter
+from dexp.utils.backends import Backend, CupyBackend, NumpyBackend
 
 
 def demo_butterworth_filter_numpy():
@@ -30,21 +28,19 @@ def _demo_butterworth_filter():
     cutoffs = 0.75
     n = 8
 
-    filtered = butterworth_filter(noisy,
-                                  shape=shape,
-                                  cutoffs=cutoffs,
-                                  cutoffs_in_freq_units=False,
-                                  order=n)
+    filtered = butterworth_filter(noisy, shape=shape, cutoffs=cutoffs, cutoffs_in_freq_units=False, order=n)
 
     from napari import Viewer, gui_qt
+
     with gui_qt():
+
         def _c(array):
             return Backend.to_numpy(array)
 
         viewer = Viewer()
-        viewer.add_image(_c(image), name='image')
-        viewer.add_image(_c(noisy), name='noisy')
-        viewer.add_image(_c(filtered), name='filtered')
+        viewer.add_image(_c(image), name="image")
+        viewer.add_image(_c(noisy), name="noisy")
+        viewer.add_image(_c(filtered), name="filtered")
 
 
 if __name__ == "__main__":
