@@ -3,10 +3,8 @@ from scipy.ndimage import convolve
 from skimage.data import camera
 from skimage.util import random_noise
 
-from dexp.processing.backends.backend import Backend
-from dexp.processing.backends.cupy_backend import CupyBackend
-from dexp.processing.backends.numpy_backend import NumpyBackend
 from dexp.processing.filters.fft_convolve import fft_convolve
+from dexp.utils.backends import Backend, CupyBackend, NumpyBackend
 
 
 def demo_fft_convolve_numpy():
@@ -33,16 +31,18 @@ def _demo_fft_convolve():
     reference_result = convolve(noisy, psf)
 
     from napari import Viewer, gui_qt
+
     with gui_qt():
+
         def _c(array):
             return Backend.to_numpy(array)
 
         viewer = Viewer()
-        viewer.add_image(_c(image), name='image')
-        viewer.add_image(_c(noisy), name='noisy')
-        viewer.add_image(_c(psf), name='psf')
-        viewer.add_image(_c(reference_result), name='reference_result')
-        viewer.add_image(_c(result), name='result')
+        viewer.add_image(_c(image), name="image")
+        viewer.add_image(_c(noisy), name="noisy")
+        viewer.add_image(_c(psf), name="psf")
+        viewer.add_image(_c(reference_result), name="reference_result")
+        viewer.add_image(_c(result), name="result")
 
 
 if __name__ == "__main__":
