@@ -1,7 +1,5 @@
-from dexp.processing.backends.backend import Backend
-from dexp.processing.backends.cupy_backend import CupyBackend
-from dexp.processing.backends.numpy_backend import NumpyBackend
 from dexp.processing.filters.kernels.gaussian import gaussian_kernel_nd
+from dexp.utils.backends import Backend, CupyBackend, NumpyBackend
 
 
 def demo_kernels_numpy():
@@ -21,12 +19,14 @@ def _demo_kernels():
     gaussian_kernel = gaussian_kernel_nd(ndim=4, size=17, sigma=2)
 
     from napari import Viewer, gui_qt
+
     with gui_qt():
+
         def _c(array):
             return Backend.to_numpy(array)
 
         viewer = Viewer()
-        viewer.add_image(_c(gaussian_kernel), name='gaussian_kernel')
+        viewer.add_image(_c(gaussian_kernel), name="gaussian_kernel")
 
 
 demo_kernels_cupy()
