@@ -1,17 +1,21 @@
 from skimage.registration import phase_cross_correlation
 
-from dexp.processing.backends.backend import Backend
-from dexp.processing.registration.model.translation_registration_model import TranslationRegistrationModel
+from dexp.processing.registration.model.translation_registration_model import (
+    TranslationRegistrationModel,
+)
 from dexp.processing.registration.translation_nd import register_translation_nd
 from dexp.utils import xpArray
+from dexp.utils.backends import Backend
 
 
-def register_translation_2d_skimage(image_a: xpArray,
-                                    image_b: xpArray,
-                                    upsample_factor: int = 16,
-                                    force_numpy: bool = False,
-                                    internal_dtype=None,
-                                    **kwargs) -> TranslationRegistrationModel:
+def register_translation_2d_skimage(
+    image_a: xpArray,
+    image_b: xpArray,
+    upsample_factor: int = 16,
+    force_numpy: bool = False,
+    internal_dtype=None,
+    **kwargs
+) -> TranslationRegistrationModel:
     """
     Registers two 2D images using just a translation-only model using skimage registration code.
     Warning: Works well unless images have lots of noise or overlap is partial.
@@ -36,8 +40,7 @@ def register_translation_2d_skimage(image_a: xpArray,
     return TranslationRegistrationModel(shift_vector=shifts, confidence=error, force_numpy=force_numpy)
 
 
-def register_translation_2d_dexp(image_a, image_b,
-                                 **kwargs) -> TranslationRegistrationModel:
+def register_translation_2d_dexp(image_a, image_b, **kwargs) -> TranslationRegistrationModel:
     """
     Registers two 2D images using just a translation-only model using dexp own registration code.
 
@@ -52,5 +55,4 @@ def register_translation_2d_dexp(image_a, image_b,
     Translation-only registration model
 
     """
-    return register_translation_nd(image_a, image_b,
-                                   **kwargs)
+    return register_translation_nd(image_a, image_b, **kwargs)
