@@ -1,18 +1,22 @@
 import ctypes
 import os
+from typing import Optional
 
 
-def set_mkl_threads(num_threads=None):
+def set_mkl_threads(num_threads: Optional[int] = None) -> None:
     try:
         if num_threads is None:
             import multiprocessing
+
             num_threads = multiprocessing.cpu_count() // 2 - 1
 
         try:
             import mkl
+
             mkl.set_num_threads(num_threads)
             return 0
-        except Exception as e:
+
+        except Exception:
             pass
             # import traceback
             # traceback.print_exc()
@@ -23,7 +27,8 @@ def set_mkl_threads(num_threads=None):
                 mkl_rt = ctypes.CDLL(name)
                 mkl_rt.mkl_set_num_threads(ctypes.byref(ctypes.c_int(num_threads)))
                 return 0
-            except Exception as e:
+
+            except Exception:
                 pass
                 # import traceback
                 # traceback.print_exc()
@@ -34,10 +39,12 @@ def set_mkl_threads(num_threads=None):
         os.environ["MKL_NUM_THREADS"] = f"{num_threads}"  # export MKL_NUM_THREADS=6
         os.environ["VECLIB_MAXIMUM_THREADS"] = f"{num_threads}"  # export VECLIB_MAXIMUM_THREADS=4
         os.environ["NUMEXPR_NUM_THREADS"] = f"{num_threads}"  # export NUMEXPR_NUM_THREADS=6
-    except Exception as e:
+
+    except Exception:
         pass
         # import traceback
         # traceback.print_exc()
         # print(e)
 
+
 # set_mkl_threads(8)
