@@ -1,9 +1,8 @@
-from dexp.processing.backends.backend import Backend
-from dexp.processing.backends.cupy_backend import CupyBackend
-from dexp.processing.backends.numpy_backend import NumpyBackend
+from dexp.utils import xpArray
+from dexp.utils.backends import Backend, CupyBackend, NumpyBackend
 
 
-def nan_to_zero(array, copy=True):
+def nan_to_zero(array: xpArray, copy: bool = True) -> xpArray:
     """
     Replaces every nan in an array to zero. It might, or not, be able to operate in-place.
     To be safe, the returned array should always be used...
@@ -18,6 +17,7 @@ def nan_to_zero(array, copy=True):
     Array for which NaNs have been replace by zero.
 
     """
+    # TODO: should we remove this function?
     backend = Backend.current()
 
     if type(backend) is NumpyBackend:
@@ -25,4 +25,5 @@ def nan_to_zero(array, copy=True):
         return xp.nan_to_num(array, copy=copy)
     elif type(backend) is CupyBackend:
         import cupy
+
         return cupy.nan_to_num(array)
